package com.jianshui.web.controller.invoice;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jianshui.common.annotation.Log;
import com.jianshui.common.core.controller.BaseController;
import com.jianshui.common.core.domain.AjaxResult;
import com.jianshui.common.enums.BusinessType;
import com.jianshui.invoice.domain.Invoice;
import com.jianshui.invoice.service.IInvoiceListService;
import com.jianshui.common.utils.poi.ExcelUtil;
import com.jianshui.common.core.page.TableDataInfo;

/**
 * 已开发票列表Controller
 * 
 * @author Gong Quanlin
 * @date 2022-03-08
 */
@RestController
@RequestMapping("/invoice/invoice")
public class InvoiceListController extends BaseController
{
    @Autowired
    private IInvoiceListService invoiceService;

    /**
     * 查询已开发票列表列表
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:list')")
    @GetMapping("/list")
    public TableDataInfo list(Invoice invoice)
    {
        startPage();
        List<Invoice> list = invoiceService.selectInvoiceList(invoice);
        return getDataTable(list);
    }

    /**
     * 导出已开发票列表列表
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:export')")
    @Log(title = "已开发票列表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Invoice invoice)
    {
        List<Invoice> list = invoiceService.selectInvoiceList(invoice);
        ExcelUtil<Invoice> util = new ExcelUtil<Invoice>(Invoice.class);
        util.exportExcel(response, list, "已开发票列表数据");
    }

    /**
     * 获取已开发票列表详细信息
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(invoiceService.selectInvoiceById(id));
    }

    /**
     * 新增已开发票列表
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:add')")
    @Log(title = "已开发票列表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Invoice invoice)
    {
        return toAjax(invoiceService.insertInvoice(invoice));
    }

    /**
     * 修改已开发票列表
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:edit')")
    @Log(title = "已开发票列表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Invoice invoice)
    {
        return toAjax(invoiceService.updateInvoice(invoice));
    }

    /**
     * 删除已开发票列表
     */
    @PreAuthorize("@ss.hasPermi('invoice:invoice:remove')")
    @Log(title = "已开发票列表", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(invoiceService.deleteInvoiceByIds(ids));
    }
}
