package com.jianshui.storage.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.jianshui.common.core.domain.AjaxResult;
import com.jianshui.common.exception.jianshui.JianshuiServiceException;
import com.jianshui.common.utils.DictUtils;
import com.jianshui.common.utils.StringUtils;
import com.jianshui.common.utils.sign.Md5Utils;
import com.jianshui.common.utils.uuid.IdUtils;
import com.jianshui.storage.domain.StorageReesultDTO;
import com.jianshui.storage.domain.StorageUrlDTO;
import com.jianshui.storage.service.StorageBaseAbstractService;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpMethodName;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.model.ObjectMetadata;
import com.qcloud.cos.model.PutObjectRequest;
import com.qcloud.cos.model.PutObjectResult;
import com.qcloud.cos.region.Region;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description 腾讯云 文档地址：https://cloud.tencent.com/document/product/436/10199#.E5.88.9D.E5.A7.8B.E5.8C.96.E5.AE.A2.E6.88.B7.E7.AB.AF
 * @Author 巩权林
 * @Date 2022-09-15 11:58
 **/
@Service("storage_cos")
public class CosStorageService extends StorageBaseAbstractService {

    private String bucketName;

    // @Override
    // // public AjaxResult init() {
    //
    //     String appIdRaw = DictUtils.getDictValue("storage_cos", "appId");
    //     if (appIdRaw == null) {
    //         throw new JianshuiServiceException("未配置对象存储appId");
    //     }
    //     Long appId = Long.parseLong(appIdRaw);
    //     String secretId = DictUtils.getDictValue("storage_cos", "secretId");
    //     if (StringUtils.isEmpty(secretId)) {
    //         throw new JianshuiServiceException("对象存储secretId为空！");
    //     }
    //     String secretKey = DictUtils.getDictValue("storage_cos", "secretKey");
    //     if (StringUtils.isEmpty(secretKey)) {
    //         throw new JianshuiServiceException("对象存储secretKey为空！");
    //     }
    //
    //     // 设置要操作的bucket
    //     this.bucketName = DictUtils.getDictValue("storage_cos", "bucketName");
    //     if (StringUtils.isEmpty(this.bucketName)) {
    //         throw new JianshuiServiceException("对象存储未配置bucket!");
    //     }
    //     // 初始化密钥信息
    //     Credentials cred = new Credentials(appId, secretId, secretKey);
    //
    //     // 初始化客户端配置
    //     ClientConfig clientConfig = new ClientConfig();
    //     // 设置bucket所在的区域，例如华南园区：gz； 华北园区：tj；华东园区：sh ；
    //     clientConfig.setRegion("gz");
    //
    //     // 初始化cosClient
    //     this.storageInstance = new COSClient(clientConfig, cred);
    //
    //     return AjaxResult.success();
    // }

    @Override
    public AjaxResult init() {
        // 1 初始化用户身份信息（secretId, secretKey）。
        // SECRETID和SECRETKEY请登录访问管理控制台 https://console.cloud.tencent.com/cam/capi 进行查看和管理

        String appIdRaw = DictUtils.getDictValue("storage_cos", "appId");
        if (appIdRaw == null) {
            throw new JianshuiServiceException("未配置对象存储appId");
        }
        Long appId = Long.parseLong(appIdRaw);
        String secretId = DictUtils.getDictValue("storage_cos", "secretId");
        if (StringUtils.isEmpty(secretId)) {
            throw new JianshuiServiceException("对象存储secretId为空！");
        }

        String secretKey = DictUtils.getDictValue("storage_cos", "secretKey");
        if (StringUtils.isEmpty(secretKey)) {
            throw new JianshuiServiceException("对象存储secretKey为空！");
        }

        String regionRaw = DictUtils.getDictValue("storage_cos", "region");
        if (StringUtils.isEmpty(secretKey)) {
            throw new JianshuiServiceException("对象存储未配置region！");
        }

            // 设置要操作的bucket
        this.bucketName = DictUtils.getDictValue("storage_cos", "bucketName");
        if (StringUtils.isEmpty(this.bucketName)) {
            throw new JianshuiServiceException("对象存储未配置bucket!");
        }

        COSCredentials cred = new BasicCOSCredentials(secretId, secretKey);
        // 2 设置 bucket 的地域, COS 地域的简称请参照 https://cloud.tencent.com/document/product/436/6224
        // clientConfig 中包含了设置 region, https(默认 http), 超时, 代理等 set 方法, 使用可参见源码或者常见问题 Java SDK 部分。
        Region region = new Region(regionRaw);
        ClientConfig clientConfig = new ClientConfig(region);
        // 这里建议设置使用 https 协议
        // 从 5.6.54 版本开始，默认使用了 https
        clientConfig.setHttpProtocol(HttpProtocol.https);
        // 3 生成 cos 客户端。

        this.storageInstance = new COSClient(cred, clientConfig);
        return AjaxResult.success();
    }

    @Override
    public AjaxResult getFile(StorageUrlDTO storageUrlDTO) {
        String key = storageUrlDTO.getStoragePath();

        // 设置签名过期时间(可选), 若未进行设置则默认使用 ClientConfig 中的签名过期时间(1小时)
        // 这里设置签名在24h后过期
        Date expirationDate = new Date(System.currentTimeMillis() + 24 * 60 * 60 * 1000);

        // 填写本次请求的参数，需与实际请求相同，能够防止用户篡改此签名的 HTTP 请求的参数
        Map<String, String> params = new HashMap<String, String>();
        // params.put("param1", "value1");

        // 填写本次请求的头部，需与实际请求相同，能够防止用户篡改此签名的 HTTP 请求的头部
        Map<String, String> headers = new HashMap<String, String>();
        // headers.put("header1", "value1");

        // 请求的 HTTP 方法，上传请求用 PUT，下载请求用 GET，删除请求用 DELETE
        HttpMethodName method = HttpMethodName.GET;

        URL url = getClient().generatePresignedUrl(bucketName, key, expirationDate, method, headers, params);

        return AjaxResult.success("success", url.toString());
    }

    private COSClient getClient() {
        return (COSClient) this.storageInstance;
    }

    @Override
    public AjaxResult upload(byte[] data, String storagePath) {
        if (!this.hasInit) {
            throw new JianshuiServiceException("对象存储未初始化");
        }

        String key = IdUtils.fastSimpleUUID();
        InputStream inputStream = new ByteArrayInputStream(data);

        ObjectMetadata objectMetadata = new ObjectMetadata();
        // 上传的流如果能够获取准确的流长度，则推荐一定填写 content-length
        // 如果确实没办法获取到，则下面这行可以省略，但同时高级接口也没办法使用分块上传了
        objectMetadata.setContentLength(data.length);
        PutObjectRequest putObjectRequest = new PutObjectRequest(this.bucketName, key, inputStream, objectMetadata);
        try {
            PutObjectResult putObjectResult = getClient().putObject(putObjectRequest);
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }

        StorageUrlDTO storageUrlDTO = new StorageUrlDTO();
        storageUrlDTO.setStoragePath(key);
        storageUrlDTO.setServiceProvider("cos");
        return AjaxResult.success(storageUrlDTO);
    }

    @Override
    public AjaxResult upload(String filePath, String storagePath) {
        if (!this.hasInit) {
            throw new JianshuiServiceException("对象存储未初始化");
        }

        // 指定要上传的文件
        File localFile = new File(filePath);
        // 指定文件将要存放的存储桶
        String bucketName = this.bucketName;
        // 指定文件上传到 COS 上的路径，即对象键。例如对象键为folder/picture.jpg，则表示将文件 picture.jpg 上传到 folder 路径下
        String key = IdUtils.fastSimpleUUID();
        PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, key, localFile);
        try {
            PutObjectResult putObjectResult = getClient().putObject(putObjectRequest);
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }

        StorageUrlDTO storageUrlDTO = new StorageUrlDTO();
        storageUrlDTO.setStoragePath(key);
        storageUrlDTO.setServiceProvider("cos");
        return AjaxResult.success(storageUrlDTO);
    }
}
